@@ -21,14 +21,13 @@ import (
 	"net/http"
 	"testing"
 
+	"github.com/Azure/azure-sdk-for-go/services/compute/mgmt/2019-07-01/compute"
 	"github.com/Azure/go-autorest/autorest"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/mock"
 
 	apiv1 "k8s.io/api/core/v1"
 	"k8s.io/autoscaler/cluster-autoscaler/cloudprovider"
-
-	"github.com/Azure/azure-sdk-for-go/services/compute/mgmt/2018-10-01/compute"
 )
 
 func newTestScaleSet(manager *AzureManager, name string) *ScaleSet {
@@ -36,9 +35,10 @@ func newTestScaleSet(manager *AzureManager, name string) *ScaleSet {
 		azureRef: azureRef{
 			Name: name,
 		},
-		manager: manager,
-		minSize: 1,
-		maxSize: 5,
+		manager:           manager,
+		minSize:           1,
+		maxSize:           5,
+		sizeRefreshPeriod: defaultVmssSizeRefreshPeriod,
 	}
 }
 
@@ -105,6 +105,9 @@ func TestBelongs(t *testing.T) {
 
 	scaleSet, ok := provider.NodeGroups()[0].(*ScaleSet)
 	assert.True(t, ok)
+	// TODO: this should call manager.Refresh() once the fetchAutoASG
+	// logic is refactored out
+	provider.azureManager.regenerateCache()
 
 	invalidNode := &apiv1.Node{
 		Spec: apiv1.NodeSpec{
@@ -147,6 +150,9 @@ func TestDeleteNodes(t *testing.T) {
 	}
 	scaleSetClient.On("DeleteInstances", mock.Anything, "test-asg", mock.Anything, mock.Anything).Return(response, nil)
 	manager.azClient.virtualMachineScaleSetsClient = scaleSetClient
+	// TODO: this should call manager.Refresh() once the fetchAutoASG
+	// logic is refactored out
+	manager.regenerateCache()
 
 	resourceLimiter := cloudprovider.NewResourceLimiter(
 		map[string]int64{cloudprovider.ResourceNameCores: 1, cloudprovider.ResourceNameMemory: 10000000},
@@ -157,6 +163,9 @@ func TestDeleteNodes(t *testing.T) {
 	registered := manager.RegisterAsg(
 		newTestScaleSet(manager, "test-asg"))
 	assert.True(t, registered)
+	// TODO: this should call manager.Refresh() once the fetchAutoASG
+	// logic is refactored out
+	manager.regenerateCache()
 
 	node := &apiv1.Node{
 		Spec: apiv1.NodeSpec{
@@ -193,6 +202,9 @@ func TestScaleSetNodes(t *testing.T) {
 	provider := newTestProvider(t)
 	registered := provider.azureManager.RegisterAsg(
 		newTestScaleSet(provider.azureManager, "test-asg"))
+	// TODO: this should call manager.Refresh() once the fetchAutoASG
+	// logic is refactored out
+	provider.azureManager.regenerateCache()
 	assert.True(t, registered)
 	assert.Equal(t, len(provider.NodeGroups()), 1)
 
