@@ -19,21 +19,19 @@ package simulator
 import (
 	"flag"
 	"fmt"
-	"math/rand"
 	"time"
 
+	"k8s.io/autoscaler/cluster-autoscaler/utils/drain"
 	"k8s.io/autoscaler/cluster-autoscaler/utils/errors"
 	"k8s.io/autoscaler/cluster-autoscaler/utils/glogx"
 	"k8s.io/autoscaler/cluster-autoscaler/utils/gpu"
 	kube_util "k8s.io/autoscaler/cluster-autoscaler/utils/kubernetes"
 	pod_util "k8s.io/autoscaler/cluster-autoscaler/utils/pod"
-	scheduler_util "k8s.io/autoscaler/cluster-autoscaler/utils/scheduler"
 	"k8s.io/autoscaler/cluster-autoscaler/utils/tpu"
 
 	apiv1 "k8s.io/api/core/v1"
 	policyv1 "k8s.io/api/policy/v1beta1"
 	"k8s.io/apimachinery/pkg/api/resource"
-	"k8s.io/kubernetes/pkg/scheduler/algorithm/predicates"
 	schedulernodeinfo "k8s.io/kubernetes/pkg/scheduler/nodeinfo"
 
 	"k8s.io/klog"
@@ -58,6 +56,48 @@ type NodeToBeRemoved struct {
 	PodsToReschedule []*apiv1.Pod
 }
 
+// UnremovableNode represents a node that can't be removed by CA.
+type UnremovableNode struct {
+	Node        *apiv1.Node
+	Reason      UnremovableReason
+	BlockingPod *drain.BlockingPod
+}
+
+// UnremovableReason represents a reason why a node can't be removed by CA.
+type UnremovableReason int
+
+const (
+	// NoReason - sanity check, this should never be set explicitly. If this is found in the wild, it means that it was
+	// implicitly initialized and might indicate a bug.
+	NoReason UnremovableReason = iota
+	// ScaleDownDisabledAnnotation - node can't be removed because it has a "scale down disabled" annotation.
+	ScaleDownDisabledAnnotation
+	// NotAutoscaled - node can't be removed because it doesn't belong to an autoscaled node group.
+	NotAutoscaled
+	// NotUnneededLongEnough - node can't be removed because it wasn't unneeded for long enough.
+	NotUnneededLongEnough
+	// NotUnreadyLongEnough - node can't be removed because it wasn't unready for long enough.
+	NotUnreadyLongEnough
+	// NodeGroupMinSizeReached - node can't be removed because its node group is at its minimal size already.
+	NodeGroupMinSizeReached
+	// MinimalResourceLimitExceeded - node can't be removed because it would violate cluster-wide minimal resource limits.
+	MinimalResourceLimitExceeded
+	// CurrentlyBeingDeleted - node can't be removed because it's already in the process of being deleted.
+	CurrentlyBeingDeleted
+	// NotUnderutilized - node can't be removed because it's not underutilized.
+	NotUnderutilized
+	// NotUnneededOtherReason - node can't be removed because it's not marked as unneeded for other reasons (e.g. it wasn't inspected at all in a given autoscaler loop).
+	NotUnneededOtherReason
+	// RecentlyUnremovable - node can't be removed because it was recently found to be unremovable.
+	RecentlyUnremovable
+	// NoPlaceToMovePods - node can't be removed because there's no place to move its pods to.
+	NoPlaceToMovePods
+	// BlockedByPod - node can't be removed because a pod running on it can't be moved. The reason why should be in BlockingPod.
+	BlockedByPod
+	// UnexpectedError - node can't be removed because of an unexpected error.
+	UnexpectedError
+)
+
 // UtilizationInfo contains utilization information for a node.
 type UtilizationInfo struct {
 	CpuUtil float64
@@ -71,16 +111,22 @@ type UtilizationInfo struct {
 
 // FindNodesToRemove finds nodes that can be removed. Returns also an information about good
 // rescheduling location for each of the pods.
-func FindNodesToRemove(candidates []*apiv1.Node, destinationNodes []*apiv1.Node, pods []*apiv1.Pod,
-	listers kube_util.ListerRegistry, predicateChecker *PredicateChecker, maxCount int,
-	fastCheck bool, oldHints map[string]string, usageTracker *UsageTracker,
+func FindNodesToRemove(
+	candidates []string,
+	destinations []string,
+	listers kube_util.ListerRegistry,
+	clusterSnapshot ClusterSnapshot,
+	predicateChecker PredicateChecker,
+	maxCount int,
+	fastCheck bool,
+	oldHints map[string]string,
+	usageTracker *UsageTracker,
 	timestamp time.Time,
 	podDisruptionBudgets []*policyv1.PodDisruptionBudget,
-) (nodesToRemove []NodeToBeRemoved, unremovableNodes []*apiv1.Node, podReschedulingHints map[string]string, finalError errors.AutoscalerError) {
+) (nodesToRemove []NodeToBeRemoved, unremovableNodes []*UnremovableNode, podReschedulingHints map[string]string, finalError errors.AutoscalerError) {
 
-	nodeNameToNodeInfo := scheduler_util.CreateNodeNameToInfoMap(pods, destinationNodes)
 	result := make([]NodeToBeRemoved, 0)
-	unremovable := make([]*apiv1.Node, 0)
+	unremovable := make([]*UnremovableNode, 0)
 
 	evaluationType := "Detailed evaluation"
 	if fastCheck {
@@ -88,64 +134,78 @@ func FindNodesToRemove(candidates []*apiv1.Node, destinationNodes []*apiv1.Node,
 	}
 	newHints := make(map[string]string, len(oldHints))
 
+	destinationMap := make(map[string]bool, len(destinations))
+	for _, destination := range destinations {
+		destinationMap[destination] = true
+	}
+
 candidateloop:
-	for _, node := range candidates {
-		klog.V(2).Infof("%s: %s for removal", evaluationType, node.Name)
+	for _, nodeName := range candidates {
+		nodeInfo, err := clusterSnapshot.NodeInfos().Get(nodeName)
+		if err != nil {
+			klog.Errorf("Can't retrieve node %s from snapshot, err: %v", nodeName, err)
+		}
+		klog.V(2).Infof("%s: %s for removal", evaluationType, nodeName)
 
 		var podsToRemove []*apiv1.Pod
-		var err error
+		var blockingPod *drain.BlockingPod
 
-		if nodeInfo, found := nodeNameToNodeInfo[node.Name]; found {
-			if fastCheck {
-				podsToRemove, err = FastGetPodsToMove(nodeInfo, *skipNodesWithSystemPods, *skipNodesWithLocalStorage,
-					podDisruptionBudgets)
-			} else {
-				podsToRemove, err = DetailedGetPodsForMove(nodeInfo, *skipNodesWithSystemPods, *skipNodesWithLocalStorage, listers, int32(*minReplicaCount),
-					podDisruptionBudgets)
-			}
-			if err != nil {
-				klog.V(2).Infof("%s: node %s cannot be removed: %v", evaluationType, node.Name, err)
-				unremovable = append(unremovable, node)
-				continue candidateloop
-			}
-		} else {
-			klog.V(2).Infof("%s: nodeInfo for %s not found", evaluationType, node.Name)
-			unremovable = append(unremovable, node)
+		if _, found := destinationMap[nodeName]; !found {
+			klog.V(2).Infof("%s: nodeInfo for %s not found", evaluationType, nodeName)
+			unremovable = append(unremovable, &UnremovableNode{Node: nodeInfo.Node(), Reason: UnexpectedError})
 			continue candidateloop
 		}
-		findProblems := findPlaceFor(node.Name, podsToRemove, destinationNodes, nodeNameToNodeInfo, predicateChecker, oldHints, newHints,
-			usageTracker, timestamp)
+
+		if fastCheck {
+			podsToRemove, blockingPod, err = FastGetPodsToMove(nodeInfo, *skipNodesWithSystemPods, *skipNodesWithLocalStorage,
+				podDisruptionBudgets)
+		} else {
+			podsToRemove, blockingPod, err = DetailedGetPodsForMove(nodeInfo, *skipNodesWithSystemPods, *skipNodesWithLocalStorage, listers, int32(*minReplicaCount),
+				podDisruptionBudgets)
+		}
+
+		if err != nil {
+			klog.V(2).Infof("%s: node %s cannot be removed: %v", evaluationType, nodeName, err)
+			if blockingPod != nil {
+				unremovable = append(unremovable, &UnremovableNode{Node: nodeInfo.Node(), Reason: BlockedByPod, BlockingPod: blockingPod})
+			} else {
+				unremovable = append(unremovable, &UnremovableNode{Node: nodeInfo.Node(), Reason: UnexpectedError})
+			}
+			continue candidateloop
+		}
+
+		findProblems := findPlaceFor(nodeName, podsToRemove, destinationMap, clusterSnapshot,
+			predicateChecker, oldHints, newHints, usageTracker, timestamp)
 
 		if findProblems == nil {
 			result = append(result, NodeToBeRemoved{
-				Node:             node,
+				Node:             nodeInfo.Node(),
 				PodsToReschedule: podsToRemove,
 			})
-			klog.V(2).Infof("%s: node %s may be removed", evaluationType, node.Name)
+			klog.V(2).Infof("%s: node %s may be removed", evaluationType, nodeName)
 			if len(result) >= maxCount {
 				break candidateloop
 			}
 		} else {
-			klog.V(2).Infof("%s: node %s is not suitable for removal: %v", evaluationType, node.Name, findProblems)
-			unremovable = append(unremovable, node)
+			klog.V(2).Infof("%s: node %s is not suitable for removal: %v", evaluationType, nodeName, findProblems)
+			unremovable = append(unremovable, &UnremovableNode{Node: nodeInfo.Node(), Reason: NoPlaceToMovePods})
 		}
 	}
 	return result, unremovable, newHints, nil
 }
 
 // FindEmptyNodesToRemove finds empty nodes that can be removed.
-func FindEmptyNodesToRemove(candidates []*apiv1.Node, pods []*apiv1.Pod) []*apiv1.Node {
-	nodeNameToNodeInfo := scheduler_util.CreateNodeNameToInfoMap(pods, candidates)
-	result := make([]*apiv1.Node, 0)
+func FindEmptyNodesToRemove(snapshot ClusterSnapshot, candidates []string) []string {
+	result := make([]string, 0)
 	for _, node := range candidates {
-		if nodeInfo, found := nodeNameToNodeInfo[node.Name]; found {
-			// Should block on all pods.
-			podsToRemove, err := FastGetPodsToMove(nodeInfo, true, true, nil)
-			if err == nil && len(podsToRemove) == 0 {
-				result = append(result, node)
-			}
-		} else {
-			// Node without pods.
+		nodeInfo, err := snapshot.NodeInfos().Get(node)
+		if err != nil {
+			klog.Errorf("Can't retrieve node %s from snapshot, err: %v", node, err)
+			continue
+		}
+		// Should block on all pods.
+		podsToRemove, _, err := FastGetPodsToMove(nodeInfo, true, true, nil)
+		if err == nil && len(podsToRemove) == 0 {
 			result = append(result, node)
 		}
 	}
@@ -217,15 +277,19 @@ func calculateUtilizationOfResource(node *apiv1.Node, nodeInfo *schedulernodeinf
 	return float64(podsRequest.MilliValue()) / float64(nodeAllocatable.MilliValue()), nil
 }
 
-// TODO: We don't need to pass list of nodes here as they are already available in nodeInfos.
-func findPlaceFor(removedNode string, pods []*apiv1.Pod, nodes []*apiv1.Node, nodeInfos map[string]*schedulernodeinfo.NodeInfo,
-	predicateChecker *PredicateChecker, oldHints map[string]string, newHints map[string]string, usageTracker *UsageTracker,
+func findPlaceFor(removedNode string, pods []*apiv1.Pod, nodes map[string]bool,
+	clusterSnapshot ClusterSnapshot, predicateChecker PredicateChecker, oldHints map[string]string, newHints map[string]string, usageTracker *UsageTracker,
 	timestamp time.Time) error {
 
-	newNodeInfos := make(map[string]*schedulernodeinfo.NodeInfo)
-	for k, v := range nodeInfos {
-		newNodeInfos[k] = v
+	if err := clusterSnapshot.Fork(); err != nil {
+		return err
 	}
+	defer func() {
+		err := clusterSnapshot.Revert()
+		if err != nil {
+			klog.Fatalf("Got error when calling ClusterSnapshot.Revert(); %v", err)
+		}
+	}()
 
 	podKey := func(pod *apiv1.Pod) string {
 		return fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)
@@ -233,39 +297,31 @@ func findPlaceFor(removedNode string, pods []*apiv1.Pod, nodes []*apiv1.Node, no
 
 	loggingQuota := glogx.PodsLoggingQuota()
 
-	tryNodeForPod := func(nodename string, pod *apiv1.Pod, predicateMeta predicates.Metadata) bool {
-		nodeInfo, found := newNodeInfos[nodename]
-		if found {
-			if nodeInfo.Node() == nil {
-				// NodeInfo is generated based on pods. It is possible that node is removed from
-				// an api server faster than the pod that were running on them. In such a case
-				// we have to skip this nodeInfo. It should go away pretty soon.
-				klog.Warningf("No node in nodeInfo %s -> %v", nodename, nodeInfo)
-				return false
-			}
-			err := predicateChecker.CheckPredicates(pod, predicateMeta, nodeInfo)
-			if err != nil {
-				glogx.V(4).UpTo(loggingQuota).Infof("Evaluation %s for %s/%s -> %v", nodename, pod.Namespace, pod.Name, err.VerboseError())
-			} else {
-				// TODO(mwielgus): Optimize it.
-				klog.V(4).Infof("Pod %s/%s can be moved to %s", pod.Namespace, pod.Name, nodename)
-				podsOnNode := nodeInfo.Pods()
-				podsOnNode = append(podsOnNode, pod)
-				newNodeInfo := schedulernodeinfo.NewNodeInfo(podsOnNode...)
-				newNodeInfo.SetNode(nodeInfo.Node())
-				newNodeInfos[nodename] = newNodeInfo
-				newHints[podKey(pod)] = nodename
-				return true
-			}
+	tryNodeForPod := func(nodename string, pod *apiv1.Pod) bool {
+		if err := predicateChecker.CheckPredicates(clusterSnapshot, pod, nodename); err != nil {
+			glogx.V(4).UpTo(loggingQuota).Infof("Evaluation %s for %s/%s -> %v", nodename, pod.Namespace, pod.Name, err.VerboseMessage())
+			return false
 		}
-		return false
+
+		klog.V(4).Infof("Pod %s/%s can be moved to %s", pod.Namespace, pod.Name, nodename)
+		if err := clusterSnapshot.AddPod(pod, nodename); err != nil {
+			klog.Errorf("Simulating scheduling of %s/%s to %s return error; %v", pod.Namespace, pod.Name, nodename, err)
+			return false
+		}
+		newHints[podKey(pod)] = nodename
+		return true
 	}
 
-	// TODO: come up with a better semi-random semi-utilization sorted
-	// layout.
-	shuffledNodes := shuffleNodes(nodes)
-
 	pods = tpu.ClearTPURequests(pods)
+
+	// remove pods from clusterSnapshot first
+	for _, pod := range pods {
+		if err := clusterSnapshot.RemovePod(pod.Namespace, pod.Name, removedNode); err != nil {
+			// just log error
+			klog.Errorf("Simulating removal of %s/%s return error; %v", pod.Namespace, pod.Name, err)
+		}
+	}
+
 	for _, podptr := range pods {
 		newpod := *podptr
 		newpod.Spec.NodeName = ""
@@ -273,26 +329,26 @@ func findPlaceFor(removedNode string, pods []*apiv1.Pod, nodes []*apiv1.Node, no
 
 		foundPlace := false
 		targetNode := ""
-		predicateMeta := predicateChecker.GetPredicateMetadata(pod, newNodeInfos)
+
 		loggingQuota.Reset()
 
 		klog.V(5).Infof("Looking for place for %s/%s", pod.Namespace, pod.Name)
 
-		hintedNode, hasHint := oldHints[podKey(pod)]
-		if hasHint {
-			if hintedNode != removedNode && tryNodeForPod(hintedNode, pod, predicateMeta) {
+		if hintedNode, hasHint := oldHints[podKey(pod)]; hasHint {
+			if hintedNode != removedNode && tryNodeForPod(hintedNode, pod) {
 				foundPlace = true
 				targetNode = hintedNode
 			}
 		}
+
 		if !foundPlace {
-			for _, node := range shuffledNodes {
-				if node.Name == removedNode {
+			for nodeName := range nodes {
+				if nodeName == removedNode {
 					continue
 				}
-				if tryNodeForPod(node.Name, pod, predicateMeta) {
+				if tryNodeForPod(nodeName, pod) {
 					foundPlace = true
-					targetNode = node.Name
+					targetNode = nodeName
 					break
 				}
 			}
@@ -306,15 +362,3 @@ func findPlaceFor(removedNode string, pods []*apiv1.Pod, nodes []*apiv1.Node, no
 	}
 	return nil
 }
-
-func shuffleNodes(nodes []*apiv1.Node) []*apiv1.Node {
-	result := make([]*apiv1.Node, len(nodes))
-	for i := range nodes {
-		result[i] = nodes[i]
-	}
-	for i := range result {
-		j := rand.Intn(len(result))
-		result[i], result[j] = result[j], result[i]
-	}
-	return result
-}
